/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ws.impl.xml.merge;

import org.ow2.util.ee.metadata.ws.api.IWebservicesCommonClassMetadata;
import org.ow2.util.ee.metadata.ws.api.IWebservicesDeployableMetadata;
import org.ow2.util.ee.metadata.ws.api.struct.IWebServiceMarker;
import org.ow2.util.ee.metadata.ws.api.xml.merge.IClassMetadataFinder;
import org.ow2.util.ee.metadata.ws.api.xml.struct.IPortComponent;
import org.ow2.util.ee.metadata.ws.api.xml.struct.IServiceImplBean;
import org.ow2.util.ee.metadata.ws.api.xml.struct.IWebserviceDescription;
import org.ow2.util.ee.metadata.ws.api.xml.struct.IWebservices;

import javax.xml.namespace.QName;

/**
 * The WebserviceMetadataMerge is responsible to merge deployment
 * descriptor's values in the metadatas.
 *
 * @author Guillaume Sauthier
 */
public class WebservicesMetadataMerger {

    /**
     * Webservices metadata.
     */
    private IWebservicesDeployableMetadata metadata;

    /**
     * The finder helping to navigate in the metadatas.
     */
    private IClassMetadataFinder finder;

    /**
     * Construct a new merger for the given metadata and the given finder.
     * @param wsMetadata target metadata
     * @param finder metadata finder
     */
    public WebservicesMetadataMerger(IWebservicesDeployableMetadata wsMetadata,
                                     IClassMetadataFinder finder) {
        this.metadata = wsMetadata;
        this.finder = finder;
    }

    /**
     * Perform the merge operation.
     * @param wsMetadata target metadata
     * @param finder metadata finder
     */
    public static void merge(final IWebservicesDeployableMetadata wsMetadata, IClassMetadataFinder finder) {
        new WebservicesMetadataMerger(wsMetadata, finder).resolve();
    }

    /**
     * Resolve the metadatas.
     */
    private void resolve() {

        IWebservices webservices = metadata.getWebservices12();

        if (webservices == null) {
            // No merge
            return;
        }

        // Iterates on ws-description
        for (IWebserviceDescription wsd : webservices.getWebserviceDescriptions()) {
            String wsdlLocation = wsd.getWsdlFile();

            // Iterates on port-component
            for (IPortComponent pc : wsd.getPortComponents()) {

                String name = pc.getName();
                IServiceImplBean sib = pc.getServiceImplBean();

                // Manage link
                String link = null;
                if (sib.getEjbLink() != null) {
                    // ejb-link
                    link = sib.getEjbLink();
                } else {
                    // servlet-link
                    link = sib.getServletLink();
                }

                IWebservicesCommonClassMetadata classMetadata = finder.findClassMetadata(link);
                if (classMetadata == null) {
                    throw new IllegalStateException("Invalid link for port-component '" + name
                            + "', component '" + link + "' not found");
                }

                // Perform the merge
                IWebServiceMarker webService = classMetadata.getWebServiceMarker();
                if (webService != null) {
                    // There is a @WebService / @WebServiceProvider annotation
                    mergeWebService(classMetadata, wsd, pc);
                }
                // TODO else create the Marker by hand (possibly in metadata-complete mode)
            }
        }
    }

    private void mergeWebService(IWebservicesCommonClassMetadata classMetadata,
                                 IWebserviceDescription webserviceDescription,
                                 IPortComponent portComponent) {

        IWebServiceMarker marker = classMetadata.getWebServiceMarker();

        // merge wsdl-file
        String wsdlFile = webserviceDescription.getWsdlFile();
        if (!isNullOrEmpty(wsdlFile)) {
            marker.setWsdlLocation(wsdlFile);
        }

        // merge wsdl-service
        QName wsdlService = portComponent.getWsdlService();
        if (wsdlService != null) {
            // update both serviceName and targetNamespace
            marker.setServiceName(wsdlService);
        }

        // merge wsdl-port
        QName wsdlPort = portComponent.getWsdlPort();
        if (wsdlPort != null) {
            // update portName
            marker.setPortName(wsdlPort);
        }

        // merge protocol-binding
        String binding = portComponent.getProtocolBinding();
        if (!isNullOrEmpty(binding)) {
            marker.setProtocolBinding(binding);
        }

        // merge enable-mtom
        boolean enabled = portComponent.isMTOMEnabled();
        marker.setEnableMtom(enabled);

        // merge service-endpoint-interface if @WebService only
        if (!marker.isWebServiceProvider()) {
            String endpointInterface = portComponent.getServiceEndpointInterface();
            if (!isNullOrEmpty(endpointInterface)) {
                marker.setEndpointInterface(endpointInterface);
            }
        }

        // merge handler-chains
        marker.setHandlerChains(portComponent.getHandlerChains());
    }

    /**
     * Tell if the value is null or empty.
     * @param value tested value
     * @return true or false
     */
    private static boolean isNullOrEmpty(String value) {
        return ((value == null) || ("".equals(value)));
    }


}
